def check(items_or_names, expected_names):
    if isinstance(items_or_names, set):
        # items_or_names is a set of strings.
        assert items_or_names == set(expected_names)
    else:
        # items_or_names is a list of Item objects.
        names = sorted(item.name for item in items_or_names)
        expected_names = sorted(expected_names)
        assert names == expected_names



import codecs
import os
import tokenize

# Encoding to use when converting input files to unicode.
ENCODING = 'utf-8'


class VultureInputException(Exception):
    pass


def format_path(path):
    if not path:
        return path
    relpath = os.path.relpath(path)
    return relpath if not relpath.startswith('..') else path


def read_file(filename):
    # Python >= 3.2
    try:
        # Use encoding detected by tokenize.detect_encoding().
        with tokenize.open(filename) as f:
            return f.read()
    except (SyntaxError, UnicodeDecodeError) as err:
        raise VultureInputException(err)
    except AttributeError:
        # tokenize.open was added in Python 3.2.
        pass

    # Python < 3.2
    try:
        with codecs.open(filename, encoding=ENCODING) as f:
            return f.read()
    except UnicodeDecodeError as err:
        raise VultureInputException(err)


class LoggingList(list):
    def __init__(self, typ, verbose):
        self.typ = typ
        self._verbose = verbose
        return list.__init__(self)

    def append(self, item):
        if self._verbose:
            print('define {0} "{1}"'.format(self.typ, item.name))
        list.append(self, item)


class LoggingSet(set):
    def __init__(self, typ, verbose):
        self.typ = typ
        self._verbose = verbose
        return set.__init__(self)

    def add(self, name):
        if self._verbose:
            print('use {0} "{1}"'.format(self.typ, name))
        set.add(self, name)



import pytest
from vulture import Vulture

from . import check


@pytest.fixture
def v():
    return Vulture(verbose=True)


def test_old_format_string(v):
    v.scan("a = 1\n'%(a)s, %(b)d' % locals()")
    check(v.defined_funcs, [])
    check(v.defined_vars, ['a'])
    check(v.used_names, ['a', 'b', 'locals'])
    check(v.unused_vars, [])


def test_new_format_string(v):
    v.scan("a = 1\n'{a}, {b}'.format(**locals())")
    check(v.defined_funcs, [])
    check(v.defined_vars, ['a'])
    check(v.used_names, ['a', 'b', 'locals'])
    check(v.unused_vars, [])



import glob
import os.path
import subprocess
import sys

from vulture import __version__

DIR = os.path.dirname(os.path.abspath(__file__))
REPO = os.path.dirname(DIR)
WHITELISTS = glob.glob(os.path.join(REPO, 'vulture', 'whitelists', '*.py'))


def call_vulture(args, **kwargs):
    core = 'vulture'
    if sys.version_info < (2, 7):
        core = 'vulture.core'
    return subprocess.call(
        [sys.executable, '-m', core] + args, cwd=REPO, **kwargs)


def get_output(args):
    child = subprocess.Popen([sys.executable, "vulture/core.py"] + args,
                             cwd=REPO, stdout=subprocess.PIPE)
    return child.communicate()[0].decode("utf-8")


def test_script_with_explicit_whitelist():
    for whitelist in WHITELISTS:
        assert call_vulture(['vulture/core.py', whitelist]) == 0


def test_script_with_implicit_whitelist():
    assert call_vulture(['vulture/core.py']) == 0


def test_script_without_whitelist():
    assert call_vulture(['vulture/core.py', '--exclude', 'whitelists']) == 1


def test_exclude():
    assert call_vulture(['vulture/core.py', '--exclude', 'core.py']) == 0


def test_missing_file():
    assert call_vulture(['missing.py']) == 1


def test_dir():
    assert call_vulture(['tests']) == 0


def test_whitelist_with_python():
    for whitelist in WHITELISTS:
        assert subprocess.call([sys.executable, whitelist], cwd=REPO) == 0


def test_pyc():
    assert call_vulture(['missing.pyc']) == 1


def test_version():
    assert call_vulture(['--version']) == 0
    assert get_output(['--version']).strip() == 'vulture ' + __version__


def test_module():
    assert call_vulture(['vulture']) == 0



from whitelist_utils import Whitelist

# NodeVisitor methods are called implicitly.
# TODO: Add missing methods.
whitelist_node_visitor = Whitelist()
whitelist_node_visitor.visit_arg
whitelist_node_visitor.visit_alias
whitelist_node_visitor.visit_Assign
whitelist_node_visitor.visit_Attribute
whitelist_node_visitor.visit_ClassDef
whitelist_node_visitor.visit_comprehension
whitelist_node_visitor.visit_For
whitelist_node_visitor.visit_FunctionDef
whitelist_node_visitor.visit_Import
whitelist_node_visitor.visit_ImportFrom
whitelist_node_visitor.visit_Name
whitelist_node_visitor.visit_Str
